  /*
  Test code for comtrolling the Solarduino V0.2
  On the I2C Bus:
    MCP4551 Digital Potentiometer (used to read voltage on battery)
    Avago ADPS-9301 Ambient Light Sensor (returns value in lux)
    DS3231 Real Time Clock (we also get temperature from this!)
 
  On the SPI Bus:
    SD Card 
    
  */
  #include <avr/sleep.h>                                       
  #include <Wire.h>      
  #include <SdFat.h>    //SD card library
  #include <SdFatUtil.h>//SC card library
  #include <MCP4511.h>  //this file contains constants used in talking to the MCP4511 digipot 
                        //the pot is used to get voltage feedback
  #include <ADPS.h>     //contains address and command bytes for controlling the Ambient Light Sensor  
  #include <DS3231.h>   //contains address and command bytes for controlling the DS3231 real time clock
  #define error(s) error_P(PSTR(s))  // store error strings in flash to save RAM
  
// SD card initialization  
  Sd2Card card;
  SdVolume volume;
  SdFile root;
  SdFile file;  
  
 // Voltage feedback circuit variables
  float Voltage;
 // Ambient Light Sensor variables
  float Lux;
  
 // Real Time Clock variables 
  byte seconds;
  byte minutes;
  byte hours;
  byte day;
  byte date;
  byte month;
  byte year;
  //DS3231 performs temperature conversion about once per minute. 
  char tempL;  //holds temperature data left of decimal
  int tempR;  //holds temperature data right of decimal
  
  int stat;   //used to debug the alarm
  
  volatile word _alarm = 0;    // this variable is incrimented in the interrupt service routine
  
  // FLAGS
  boolean debug = true;  //when true we're debugging our logger data to the terminal
  
  //  PINS
  int Trip = 9;        //used in the Voltage Feedback function
  int P_Gate = 8;      //switches power to the devices on board (1=off, 0=on)
  int A = 2;           //attached to DS3231 INT pin for regular wake from sleep
  
  void error_P(const char* str) {    // mysterious SD card error report function.
  PgmPrint("error: ");               // you do not want to see this message on the serial port.
  SerialPrintln_P(str);
  if (card.errorCode()) {
    PgmPrint("SD error: ");
    Serial.print(card.errorCode(), HEX);
    Serial.print(',');
    Serial.println(card.errorData(), HEX);
  }
}
    
  void setup() {
    pinMode(P_Gate,OUTPUT);            //drive for P-MOSFET peripherals power control
    pinMode(Trip,INPUT);               //read voltage trigger 
    pinMode(A,INPUT);                  //interrupt from RTC
    Serial.begin(9600);
    Wire.begin();                      //I2C initialization
    digitalWrite(P_Gate,LOW);         //turn on board (P-MOSFET power gate)
    delay(1000);
    Serial.println("Starting Up in T!");  //current version. reverse alpha
// Disconnect the MCP digi pot    
    I2C_Write(MCP,TCON_W,W0_DISABLE);   //disconnect the digital potentiometer (lowest power)
    Serial.println("MPC disabled");         
// Powerup ADPS
    I2C_Write(ADPS,CNTRL,3);            //power up the ambient light sensor
// Verify connection to ADPS   
    int I = I2C_Read(ADPS,ID_REVNO,1);   //get ambient light sensor part id# and revision#
    Serial.print("ADPS ID = 0x");Serial.println(I,HEX);// should print 0x50
// set Alarm 1 on DC3231 to interrupt in minutes
    set_alarm(0x10);
// enable Alarm 1 on DC3231
    enable_alarm();        
// Sleep Settings    
    set_sleep_mode(SLEEP_MODE_PWR_DOWN);     // sleep mode is set here Power Down uses the least current
                                            // system clock is turned off, so millis won't be reliable in isr!
// Initialize SD card, create file, etc.
    initialize_SD();
// Interrupt settings
  attachInterrupt(0, isr, LOW);            // low level generates interrupt on pin 2, goto isr on wake


  }
  
  void loop(){

      Lux = getLux();               //getLux returns a float
      Voltage = getVoltage();       //getVoltage returns a float
      getTimeTemp();                //getTimeTemp affects global variables
      if (_alarm > 0){reInit_SD();}
      logData();                    //write data to the SD card
      if(debug == true){serialDebug();}   //prints values to terminal window for debugging 
      
      
      Serial.println("sleeping");
      system_sleep();                // low level on pin 2 will wake from sleeP
     
  }
  
  
  

  
  
void serialDebug(){
//  Voltage feedback
     Serial.print("Voltage = ");
     Serial.println(Voltage,2);
//  Ambient light level     
     Serial.print("Lux: ");
     Serial.println(Lux,2);
//  time/date stamp     
     Serial.print(tempL,DEC); Serial.print("."); Serial.print(tempR); Serial.println("*C");
     Serial.print(hours,HEX); Serial.print(":"); Serial.print(minutes,HEX);
     Serial.print(":"); Serial.println(seconds,HEX);
     Serial.print(month,HEX); Serial.print("/");Serial.print(date,HEX);
     Serial.print("/"); Serial.println(year,HEX);
     Serial.println(); 
  
}
  
  
  

